#!/usr/bin/env python

import pygtk
import gtk
import task as t
import createdb as db
import datetime as dt
sqlcon = db.SQLConnection()

pygtk.require('2.0')

#CONFIGS
COMBO_FREQ_ITEMS = ['Hourly','Daily','Weekly','Monthly']

WEEK_DAYS = {'Monday' : 1 , 'Tuesday' : 2 , 'Wednesday' : 3 , 
             'Thurday' : 4 , 'Friday' : 5 , 'Saturday' : 6 ,
             'Sunday' : 0}

class ArkModel:

    def get_tasks(self):
        tasks = []
        rc = sqlcon.find(t.Task, None, None, False)
        
        for task in rc:
            tTransport = t.Transport(task['tas_sto_id'])
            tFrecuency = t.TemporalBasis(task['tas_teb_id'])
            
            task['frecuency'] = tFrecuency
            
            tasks.append(task)

        return tasks

    def save_task(self,**kwargs):
        
        new_transport = t.Transport(None,
                                    sto_transport =kwargs['tas_transport'] , 
                                    sto_base = kwargs['tas_base'],
                                    sto_name = 'Test')
       
        if new_transport.validate() == False:
            print 'Error al Validar transporte'

        new_transport.save()
        t.sqlcon.commit()

        new_dict = dict((x,None) for x in dir(t.Task) if x.startswith('tas_'))

        new_dict['tas_including'] = u'*'
        new_dict['tas_excluding'] = u'*.aspx'
        new_dict['tas_teb_id'] = kwargs['tas_teb_id']
        new_dict['tas_sto_id'] = new_transport
        new_dict['tas_to'] = ''
        new_dict['tas_from'] = kwargs['tas_from']
        new_dict['tas_time_coord'] = kwargs['tas_time_coord']

        new_task = t.Task(None,**new_dict)
        
        new_task.commit()


class ArkController:
    def __init__(self,model):
        self.model = model
        self.freq_hour_of_day = []
        self.freq_days_of_month = []
        self.freq_days_of_week = []

    def register_view(self,view):
        self.view = view
        self.set_connects()
        self.load_data()

    def get_view(self):
        return self.view

    def set_connects(self):
        view = self.get_view()

        #Window
        view.window.connect("delete_event" , self.delete_event)

        #Buttons
        view.boton.connect('clicked',self.search_file,view.entry_watb)
        view.boton_wetb.connect('clicked',self.search_file,view.entry_whtb)
        view.btn_save.connect('clicked',self.save)
        #view.btn_add_daily_time.connect('clicked',self.freq_add_hour_of_day)
        #view.btn_add_days_of_week.connect('clicked',self.freq_add_days_of_week)
        
        #ComboBoxe
        view.combo_wetb.connect('changed', self.change_combo_wetb)
        view.combo_freq.connect('changed', self.change_combo_freq)

        #List
        view.treeview.connect("button-press-event",self.popup_list_menu)

    def delete_event(self, widget, event, data=None):
        gtk.main_quit()        
        return False

    def freq_add_hour_of_day(self,widget,data=None):
        view = self.get_view()

        hour = view.sb_hour.get_value_as_int()
        minutes = int(self.get_combo_active(view.combo_minutes))

        self.freq_hour_of_day.append(dt.time(hour,minutes,0))

        print self.freq_hour_of_day

    def freq_add_days_of_week(self,widget):
        view = self.get_view()
        print WEEK_DAYS[self.get_combo_active(view.combo_days_of_week)]
        print self.get_combo_active(view.combo_days_of_week)
        day = WEEK_DAYS[self.get_combo_active(view.combo_days_of_week)]

        self.freq_days_of_week.append(day)

        print self.freq_days_of_week

    def freq_add_day_of_month(self,widget):
        view = self.get_view()
        
        day = int(self.get_combo_active(view.combo_days_month))

        self.freq_day_of_month.append(day)

    def clean_form(self):
        view = self.get_view()

        view.entry_whtb.set_text("")
        view.entry_watb.set_text("")
    
    def add_to_list(self, **kwargs):
        view = self.get_view()
        view.lista.append([kwargs['activo'],kwargs['path_to_backup'],
                            kwargs['frequency'],kwargs['last_backup']])
    
    def load_data(self):

        tasks = self.model.get_tasks()

        for task in tasks:             
            self.add_to_list(activo = 'Yes' ,
                             path_to_backup = task['tas_from'] , 
                             frequency = task['frecuency'].teb_name,
                             last_backup = "")
    
    def popup_list_menu(self,widget,event):
        if event.button == 3 : 
            menu = gtk.Menu()
            menu_activate = gtk.MenuItem('Desactivar')
            menu_activate.show()
            menu.append(menu_activate)
            menu.show()
            menu.popup(None, None, None, event.button, event.time)

    def change_combo_freq(self,widget,data=None):
        view = self.get_view()

    def change_combo_wetb(self,widget,data=None):
        view = self.get_view()
        index = self.get_combo_active(widget)
        
        if index == 'Local':
            view.boton_wetb.set_child_visible(1)
        else:
            view.boton_wetb.set_child_visible(0)
        return


    def get_combo_active(self,combobox):
        model = combobox.get_model()
        index = combobox.get_active()
        if index:
            return model[index][0]

    def save(self, widget,data=None):
        """Prepare everything to save the task"""
        view = self.get_view()
        
        freq = self.get_combo_active(view.combo_freq)
        
        self.add_to_list(activo = 'Yes'
                         ,path_to_backup = view.entry_watb.get_text()
                         ,frequency = freq
                         ,last_backup = None)

        #Remoto o Local
        if(self.get_combo_active(view.combo_wetb) == "Remote"):
            base = 'bzr+ssh://'
        else:
            base = 'file://'
        
        #Hardcoded
        tas_teb_id = 1

        if(freq == 'Hourly'):
            tas_teb_id = 4
        elif(freq == 'Daily'):
            tas_teb_id = 3
        elif(freq == 'Weekly'):
            tas_teb_id = 2
        elif(freq == 'Monthly'):
            tas_teb_id = 1

        #Temporal Basics
        tas_time_coord = dict()
        if (len(self.freq_hour_of_day) > 0):
            tas_time_coord['HD'] = self.freq_hour_of_day
        
        if (len(self.freq_days_of_week) > 0):
            tas_time_coord['DW'] = self.freq_days_of_week

        self.model.save_task(tas_base = base,
                             tas_transport = view.entry_whtb.get_text() , 
                             tas_from = view.entry_watb.get_text() ,
                             tas_teb_id = tas_teb_id ,
                             tas_time_coord = tas_time_coord)


        self.clean_form()

    def search_file(self,widget, data=None):

        dialog = gtk.FileChooserDialog("Open..",
                                       None,
                                       gtk.FILE_CHOOSER_ACTION_OPEN,
                                       (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                       gtk.STOCK_OPEN, gtk.RESPONSE_OK))

        dialog.set_default_response(gtk.RESPONSE_OK)
        filter = gtk.FileFilter()
        filter.set_name("All files")
        filter.add_pattern("*")
        dialog.add_filter(filter)
       
        dialog.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
        #dialog.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)

        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            data.set_text(dialog.get_filename())
            #print dialog.get_filename(), 'selected'
        #elif response == gtk.RESPONSE_CANCEL:
           
        dialog.destroy()

class ArkView:

    def _create_window(self):
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.set_title("ArkGTK")
        #self.window.set_resizable(False)
        self.window.set_position(gtk.WIN_POS_CENTER)
        self.window.set_border_width(0)
        

    def _create_menu(self):
        self.menu = gtk.Menu()
        self.root_menu = gtk.MenuItem("File")
        self.root_menu.set_submenu(self.menu)
        self.menu_bar = gtk.MenuBar()
        self.menu_bar.append (self.root_menu)


    def __init__(self,controller):

        self._create_window()
        self._create_menu()

        main_vbox = gtk.VBox()
        main_hbox = gtk.HBox()
        self.window.add(main_vbox)

        main_vbox.pack_start(self.menu_bar,False,False,0)

        main_vbox.pack_start(main_hbox,True,True)
        #Primera Columna
        vbox = gtk.VBox()
        main_hbox.pack_start(vbox,True,True)

        label = gtk.Label('What to Backup: ')        
        vbox.pack_start(label, True , True)

        label = gtk.Label("Where to backup:")
        vbox.pack_start(label, True, True)

        label = gtk.Label("Frequency")
        vbox.pack_start(label, True , True)
        
        #Segunda columna
        vbox = gtk.VBox(True)
        main_hbox.pack_start(vbox,True,True)

        self.entry_watb = gtk.Entry()
        vbox.pack_start(self.entry_watb, True , True)
        self.entry_watb.set_size_request(300,25)

        self.entry_whtb = gtk.Entry()
        vbox.pack_start(self.entry_whtb, True , True)
        self.entry_watb.set_size_request(300,25)

        self.combo_freq = gtk.combo_box_new_text()
        self.combo_freq.append_text("")
        for i in COMBO_FREQ_ITEMS:
            self.combo_freq.append_text(i)
        
        vbox.pack_start(self.combo_freq, False , False)


        #Tercera Columna
        vbox = gtk.VBox()
        main_hbox.pack_start(vbox,False,False)

        self.boton = gtk.Button('Search')
        vbox.pack_start(self.boton, False, False)
      
        self.combo_wetb = gtk.combo_box_new_text()
        self.combo_wetb.append_text("")
        self.combo_wetb.append_text("Local")
        self.combo_wetb.append_text("Remote")
        vbox.pack_start(self.combo_wetb, False, False)
        
        
        #Cuarta Columna
        vbox = gtk.VBox(True,8)
        main_hbox.pack_start(vbox,True,True)
        
        self.boton_wetb = gtk.Button('Search')
        self.boton_wetb.set_child_visible(0)
        vbox.pack_start(self.boton_wetb, False , False)

       
        #FREQUENCY CONFIGS
        """
        vbox = gtk.VBox(False, 0)
        self.hbox3.pack_start(vbox,False,False)

        hbox = gtk.HBox(False, 0)
        vbox.pack_start(hbox, False,False, 5)

        vbox2 = gtk.VBox(False, 0)
        hbox.pack_start(vbox2, False, False, 5)

        label = gtk.Label("Hour :")
        label.set_alignment(0, 0.5)
        vbox2.pack_start(label,False, False, 0)
                                          
        adj = gtk.Adjustment(1.0, 1.0, 24.0, 1.0, 5.0, 0.0)
        self.sb_hour = gtk.SpinButton(adj, 0, 0)
        self.sb_hour.set_wrap(True)
        vbox2.pack_start(self.sb_hour, False, True, 0)      
        
        vbox2 = gtk.VBox(False, 0)
        hbox.pack_start(vbox2, True, True, 5)
 
        label = gtk.Label("Minutes :")
        label.set_alignment(0, 0.5)
        vbox2.pack_start(label,False, False, 0)
 
        self.combo_minutes = gtk.combo_box_new_text()
        mins=5
        while(mins <= 45):
            self.combo_minutes.append_text(str(mins))
            mins += 5
        vbox2.pack_start(self.combo_minutes, False, True, 0)      
        
        vbox2 = gtk.VBox(False,0)
        hbox.pack_start(vbox2,False,False)

        self.btn_add_daily_time = gtk.Button('[+]')
        vbox2.pack_start(self.btn_add_daily_time)

        #Day of weeks

        #vbox2 = gtk.VBox()
        #hbox.pack_start(vbox2,False,False)
        
        label = gtk.Label("Day of Week :")
        label.set_alignment(0, 0.5)
        vbox2.pack_start(label,False, False, 0)

        
        self.combo_days_of_week = gtk.combo_box_new_text()
        for i in WEEK_DAYS:
            self.combo_days_of_week.append_text(i)
          
        vbox2.pack_start(self.combo_days_of_week, False, True, 0)      
        
        vbox2 = gtk.VBox(False,0)
        hbox.pack_start(vbox2,False,False)

        self.btn_add_days_of_week = gtk.Button('[++]')
        vbox2.pack_start(self.btn_add_days_of_week,False,False)

        #self.hbox3.set_child_visible(0)
        """
        #SAVE BOX
        self.btn_save = gtk.Button('Save!')
        main_vbox.pack_start(self.btn_save, True , True)
        
        #List
        self.lista = gtk.ListStore(str, str, str, 'gboolean')
        self.lista.height = 150

        self.treeview = gtk.TreeView(self.lista)        
        self.treeview.set_size_request(500,200)
        
        lista_col_1 = gtk.TreeViewColumn('Active')
        lista_col_2 = gtk.TreeViewColumn('/path/al/backup')
        lista_col_3 = gtk.TreeViewColumn('Frequency')
        lista_col_4 = gtk.TreeViewColumn('Last Backup Date') 

        self.treeview.append_column(lista_col_1)
        self.treeview.append_column(lista_col_2)
        self.treeview.append_column(lista_col_3)
        self.treeview.append_column(lista_col_4)

        #cell = gtk.CellRendererText
        cell_1 = gtk.CellRendererText()
        cell_2 = gtk.CellRendererText()
        cell_3 = gtk.CellRendererText()
        cell_4 = gtk.CellRendererText()

        lista_col_1.pack_start(cell_1, False)
        lista_col_2.pack_start(cell_2, False)
        lista_col_3.pack_start(cell_3, False)
        lista_col_4.pack_start(cell_4, False)

        lista_col_1.set_attributes(cell_1, text=0)
        lista_col_2.set_attributes(cell_2, text=1)
        lista_col_3.set_attributes(cell_3, text=2)
        lista_col_4.set_attributes(cell_4, text=3)
        
        self.treeview.set_search_column(0)
       
        #SCROLLED
        self.scrolledwindow = gtk.ScrolledWindow()
        self.scrolledwindow.add(self.treeview)
        self.scrolledwindow.set_size_request(700,200)
        main_vbox.pack_start(self.scrolledwindow, True, True)

        self.window.show_all()

        controller.register_view(self)


def main():
    model = ArkModel()
    controller = ArkController(model)
    view = ArkView(controller)
    gtk.main()

    return

if __name__ == '__main__':
    main()  
