# Copyright (C) 2008 Martin Albisetti

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import task as t
# An instance of Transport can be generated in 2 different ways:

# === 1 ===
# creating a new transport:

ssh_tr = 'mariano@10.23.1.3/home/mariano'
dir_tr = '/home/mariano/Downloads'

# for ssh
# as you can see, the first parameter is null (you only use it when you grab an
# existing transport from the database and you need to supply your user
# password to validate the access. You need to specify a name for the
# transport.
tr = t.Transport(None, ssh_tr, 'this is my new transport', 'your_password')
# once the instance exists, you should validate it to verify it's ok
tr.validate()
# if it's ok, save it
tr.save()
# IMPORTANT: you need to commit your change after saving. This is an intended
# behaviour to prevent a transport commiting on going changes in a Task
# creation scenario
t.sqlcon.commit()
# now the same but with a dir
del tr
tr = t.Transport(None, dir_tr, 'No access Transport')
tr.validate()
tr.save()
t.sqlcon.commit()
# you cannot use a dir where you have no write access
"""
>>> tr = t.Transport(None, '/usr/bin')
>>> tr.validate()
Traceback (most recent call last):
      File "<stdin>", line 1, in <module>
        File "task.py", line 456, in validate
            access for the base))
            task.TransportError: You need read/write access for the base
"""
# you cannot use it on a symlink (NotImplementedError) or on a file either.
# don't miss the ssh pass either!


# === 2 ===
# retrieving an existing transport from the database:
# first option: use the sto_id (you need to know it)
"""
>>> tr = t.Transport(2)
>>> type(tr)
<class 'task.Transport'>
>>> tr.sto_transport, tr.sto_base
(u'bzr+ssh://', u'mmara@10.23.1.2/home/mmara')
"""
# second option: use the base instead of the id
"""
>>> del tr
>>> tr = t.Transport(None, 'mmara@10.23.1.2/home/mmara')
>>> tr.sto_transport, tr.sto_base
(u'bzr+ssh://', u'mmara@10.23.1.2/home/mmara')
"""

# You can cancel a given transport if you want
tr = t.Transport(3)
tr.cancel()
qlcon.commit()
"""
>>> tr = t.Transport(3)
Traceback (most recent call last):
      File "<stdin>", line 1, in <module>
        File "task.py", line 417, in __init__
            cancelled))
            task.TransportError: The selected transport is cancelled

>>> tr = t.Transport(None, 'mariano@10.23.1.3/home/mariano')
Traceback (most recent call last):
.     File "<stdin>", line 1, in <module>
        File "task.py", line 428, in __init__
            cancelled))
            task.TransportError: The selected transport is cancelled
"""
