#! /usr/bin/env python
# Copyright (C) 2008 Martin Albisetti

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""FIXME: write module's __doc__"""

# standard library imports
import tempfile
from optparse import OptionParser
import shutil
import os

# related 3rd party imports

# local application/ library specific imports.
from rdiff_backup import librsync, Rdiff, rpath, Globals

def main():
    parser = OptionParser(description="Companion script for Backup Ark""")
    parser.add_option("-o", "--original_file", dest="orf", help="""Name of the\
    file to be updated""")
    parser.add_option("-d", "--delta_file", dest="dfi", help="""Name of the\
    modified file's delta""")
    parser.add_option("-n", "--new_delta", dest="ndn", help="""Name of the new \
    delta file to be generated""")
    parser.add_option("-c", "--compress_method", dest="com", help="""Selected \
    compress method""")
    (options, args) = parser.parse_args()
    if not options.com:
        comp_method = None
    else:
        comp_method = options.com
    update_file(options.orf, options.dfi, comp_method)
    delta_forward("%s.tmp" % (options.orf), "%s.sig" % (options.orf),
                  options.ndn, comp_method)

class BumRdiffError(Exception):
    pass

def __compress(stream, method=None):
    if not method:
        return stream
    else:
        return stream.encode(method)

def __decompress(stream, method=None):
    if not method:
        return stream
    else:
        return stream.decode(method)

def sign(infile, compression_method=None):
    """Given a file-like object will get its signature and return it.
    The object will be closed after calculating the signature
    Parameters:
        infile: the file-like object
        compression_method: the compression algorithm used to return the
        result. It supports all codecs supported by Python. Options are: zip,
        bz2 and None.
    """
    bs = Rdiff.find_blocksize(os.path.getsize(infile.name))
    re = librsync.SigFile(infile, bs)
    while not re.eof:
        yield __compress(re.read(Globals.blocksize), compression_method)

def delta(signature, file_, compression_method=None):
    """Generator to obtain the diff between a file and its counterpart's
    signature. File will be closed when its done
    Parameters:
        file_: the file-like object
        compression_method: the compression algorithm used to return the
        result. It supports all codecs supported by Python. Options are: zip,
        bz2 and None.
    """
    delta = librsync.DeltaFile(signature, file_)
    while not delta.eof:
        yield __compress(delta.read(Globals.blocksize), compression_method)

def patch(basis_file, delta_file):
    """Generator to obtain the results of patching a file"""
    pf = librsync.PatchedFile(basis_file, delta_file)
    while not pf.eof:
        try:
            yield pf.read(Globals.blocksize)
        except librsync.librsyncError, errmsg:
            #log.error(errmsg)
            raise BumRdiffError(errmsg)

def update_file(basis_file, delta_file, compress_method):
    """FIXME: __doc__"""
    # First we build a copy of the original file:
    shutil.copy(basis_file, "%s.tmp" % (basis_file))
    to_patch = open(basis_file, 'wb')
    patch_d = open(delta_file, 'rb')
    #sigtmp, signam = tempfile.mkstemp()
    decompress_delta = open("decompress_delta", "wb")
    data = ""
    while True:
        data = patch_d.read(Globals.blocksize)
        if data == "": break
        decompress_delta.write(__decompress(data, compress_method))
        #os.write(sigtmp, __decompress(data))
    #os.close(sigtmp)
    decompress_delta.close()
    patch_d.close()
    #os.remove(patch_d)
    sigtmp = open("decompress_delta", 'rb')
    # now patch the original file with the delta
    or_file = open("%s.tmp" % (basis_file), "rb")
    for every in patch(or_file, sigtmp):
        to_patch.write(every)
    sigtmp.close()
    to_patch.close()
    to_patch = open(basis_file, 'rb')
    #os.remove(signam)
    # write the sig of the new file:
    sigf = open("%s.sig" % (basis_file), 'wb')
    for every in sign(to_patch, None):
        sigf.write(every)
    sigf.close()

def delta_forward(original_file, sig_file, delta_name, compress_method):
    """FIXME: __doc__"""
    orig = open(original_file, 'rb')
    sig = open(sig_file, 'rb')
    deltaf = open(delta_name, 'wb')
    for every in delta(sig, orig, compress_method):
        deltaf.write(every)
    deltaf.close()
    sig.close()
    orig.close()
    os.remove(original_file)

if __name__ == '__main__':
    main()
