#! /usr/bin/env python2.5
# Copyright (C) 2008 Martin Albisetti 

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


"""Testing createdb"""

# standard library imports
import unittest
import sys
import sqlite3 as sql
import datetime as dt
# related 3rd party imports
import coverage
# local application/ library specific imports.
sys.path.insert(1,sys.path[0].rsplit('/',1)[0])
import task as t 
import createdb as db


class testCreateDB(unittest.TestCase):

    def testSelectAsterisk(self):
        """We select all values from a table using the __init__ from a 
        selected class"""
        tb = t.TemporalBasis(1)
        self.failUnlessEqual(isinstance(tb, t.TemporalBasis), True, 
        """The object is not an instance of the requested class""")
        self.failUnlessEqual(tb.teb_name, u"MONTHLY", 
        """The requested value is wrong""")

    def testSelectSomeCols(self):
        """We select a few columns to build an object using the method
        directly"""
        sqlcon = db.SQLConnection()
        vnow = dt.datetime.now()
        sqlcon.cursor.execute("""insert into task (tas_from, tas_teb_id, 
        tas_add_date) values (?, ?, ?)""", ('/tmp', 1, vnow))
        sqlcon.cursor.execute("""select tas_id from task where 
        tas_add_date=?""", (vnow,))
        row = sqlcon.cursor.fetchone()
        ta = t.Task()
        try:
            rs = sqlcon.get(ta, row[0], tas_from='from_', 
            tas_add_date='add_date')
            ta.tas_from = rs['from_']
            ta.tas_add_date = rs['add_date']
        except sql.OperationalError:
            sqlcon._debug = True
            rs = sqlcon.get(ta, row[0], tas_from='from_', 
            tas_add_date='add_date')
        sqlcon.cursor.execute("""delete from task""")
        sqlcon.commit()
        sqlcon.disconnects()

def SuiteCD():
    suite = unittest.TestLoader().loadTestsFromTestCase(testCreateDB)
    return suite

if __name__ == '__main__':
    runner = unittest.TextTestRunner()
    loader = unittest.TestLoader()
    results = unittest.TestResult()
    suite = SuiteCD()
    coverage.erase()
    coverage.exclude("createdb.createdb")
    coverage.start()
    results = runner.run(suite)
    coverage.stop()
    coverage.report('../createdb.py')
    coverage.erase()
