#! /usr/bin/env python2.5
# Copyright (C) 2008 Martin Albisetti 

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


"""Testing task.Task"""

# standard library imports
import unittest
import sys
import datetime as dt
import random
import sqlite3 as sql
import glob
import os
# related 3rd party imports
import coverage
# local application/ library specific imports.
sys.path.insert(1,sys.path[0].rsplit('/',1)[0])
import task as t 
import createdb as db

sqlcon = db.SQLConnection()

class testError(Exception):
    pass

class testTask(unittest.TestCase):

    _dont_delete = False
    tas_id = None

    def test0TaskLoading(self):
        """We create a new full task"""
        add_date = dt.datetime.now()
        cancel_date = None
        from_ = unicode(os.getcwd()) 
        including = [u'*.py']
        excluding = []
        max = (12000,12000)
        tb = t.TemporalBasis(1)
        ts = t.TaskSchedule(None, tb, DM=[1,15,21])
        ta = t.Task(None, tas_add_date=add_date, 
        tas_cancel_date=cancel_date, tas_from=from_, 
        tas_including=including, tas_excluding=excluding,
        tas_max=max, tb=tb, ts=ts)
        self.failUnlessEqual(isinstance(ta, t.Task), True, """We created \
a Task object with failure""")
        self.tas_id = ta.commit()

    def test1TaskExists(self):
        """We load an already existing task using the tas_id"""
        ta = t.Task(1)

    def test1TaskNotExists(self):
        """We load an unexisting task"""
        self.failUnlessRaises(db.SQLError, t.Task, 2)

    def test2CancelTask1(self):
        """We will cancel a loaded task and verify we can't do anything else
        with it, without sending it the to the database"""
        got_error = False
        ta = t.Task(1)
        ta.tas_cancel_date = dt.datetime.now()
        try:
            ta.tas_from = u'%s/tests' % (os.getcwd())
        except t.Task.ReadOnlyError:
            got_error = True
        self.failUnlessEqual(got_error, True, """No Task.ReadOnlyError was \
raised""")

    def test2CancelTask2(self):
        """We will cancel a loaded task and verify we can't do anything else
        with it unless we reanimate it"""
        got_error = False
        ta = t.Task(1)
        ta.tas_cancel_date = dt.datetime.now()
        try:
            ta.tas_from = u'%s/tests' % (os.getcwd())
        except t.Task.ReadOnlyError:
            got_error = True
        self.failUnlessEqual(got_error, True, """No Task.ReadOnlyError was \
raised""")
        ta.tas_cancel_date = None
        ta.tas_from = u'%s/tests' % (os.getcwd())

    #def testTaskExistsQuery(self):
    #    """We load already existing task using tas_cancel_date is Null."""
    #    pass

    #def testTaskExistsMonthly(self):
    #    """We load an existing Monthly task, load its dict and request the
    #    next 10 scheduled executions"""
    #    pass

    #def testTaskExistsWeekly(self):
    #    """We load an existing Weekly task, load its dict and request the
    #    next 10 scheduled executions"""
    #    pass

    #def testTaskExistsDaily(self):
    #    """We load an existing Daily task, load its dict and request the
    #    next 10 scheduled executions"""
    #    pass

    #def testTaskExistsHourly(self):
    #    """We load an existing Hourly task, load its dict and request the
    #    next 10 scheduled executions"""
    #    pass

    #def testTaskChangePeriod1(self):
    #    """We load an existing task and change its scheduling periods"""
    #    pass

    #def testTaskChangePeriod2(self):
    #    """We load an existing task and change its scheduling periods"""
    #    pass

    #def testTaskChangePeriod3(self):
    #    """We load an existing task and change its scheduling periods"""
    #    pass

    #def testTaskChangePeriod4(self):
    #    """We load an existing task and change its scheduling periods"""
    #    pass

    #def testTaskChangePeriod5(self):
    #    """We load an existing task and change its scheduling periods"""
    #    pass

    #def testTaskWrongPath(self):
    #    """We create a task but the to-backup path does not exist"""
    #    pass

    #def testTaskWrongTemporalBasis(self):
    #    """We create a Task with an unexisting temporal basis """
    #    pass

    #def testTaskSelfExcludingPath(self):
    #    """We create a task but the to-backup path is in Exclude too"""
    #    pass

    #def testTaskCancelwithPending(self):
    #    """We cancel a task that already has a few scheduled executions"""
    #    pass

    #def testTaskLoadSuccessfully(self):
    #    """We create a Task with all ok parameters"""
    #    pass

    #def testTaskLoadWrongTemporalBasis(self):
    #    """We create a task with a wrong temporal_basis object"""
    #    pass

    #def testTaskSSH(self):
    #    """We create a task and SSH is the selected transport"""

    #def testTaskUSB(self):
    #    """We create a task and USB is the selected transport"""
    #    pass
    

def SuiteTAS():
    suite = unittest.TestLoader().loadTestsFromTestCase(testTask)
    return suite

if __name__ == '__main__':
    runner = unittest.TextTestRunner()
    loader = unittest.TestLoader()
    results = unittest.TestResult()
    suite = SuiteTAS()
    coverage.erase()
    coverage.start()
    results = runner.run(suite)
    coverage.stop()
    coverage.report('../task.py')
    coverage.erase()
    sqlcon.cursor.execute("""delete from watched_file""")
    sqlcon.cursor.execute("""delete from task_schedule""")
    sqlcon.cursor.execute("""delete from task""")
    sqlcon.commit()
    sqlcon.disconnects()


