#! /usr/bin/env python2.5
# Copyright (C) 2008 Martin Albisetti 

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


"""Testing task.TemporalBasis"""

# standard library imports
import unittest
import random as ran
import sys
# related 3rd party imports
import coverage
import glob
# local application/ library specific imports.
sys.path.insert(1,sys.path[0].rsplit('/',1)[0])
from task import TemporalBasis
import createdb as db

class testTemporalBasis(unittest.TestCase):

    def setUp(self):
        self.store = db.SQLConnection()
        self.store.cursor.execute("select * from temporal_basis""")
        rs = self.store.cursor.fetchall()
        lis = [val[0] for val in rs]
        self.valid_id = ran.choice(lis)
        self.invalid_id = ran.randint(lis[-1]+1, lis[-1]+100)
        
    def testIdExists(self):
        """We're creating an instance of TemporalBasis with a valid p_id""" 
        valid_inst = TemporalBasis(self.valid_id)
        self.failIfEqual(isinstance(valid_inst, TemporalBasis), False, 
        """Init with valid temporal_basis.teb_id but the object is not an \
instance of TemporalBasis""")

    def testIdNoExist(self):
        """We're creating an instance of TemporalBasis with an invalid p_id"""
        self.failUnlessRaises(db.SQLError, TemporalBasis, self.invalid_id)

    def testUserSetter(self):
        """An user should not set an attribute bypassing the __init__"""
        checked = False
        try:
            user_inst = TemporalBasis()
            user_inst.teb_name = "Maradona"
        except:
            checked = True
        self.failUnless(checked==True, """User could set a TemporalBasis \
attribute without init""")

    def testBetterErrorMessage(self):
        """We create an instance of TemporalBasis with an invalid id and
        we expect Ghost's own error message and not a generic one"""
        self.failUnlessRaises(db.SQLError, TemporalBasis, self.invalid_id)

def SuiteTB():
    suite = unittest.TestLoader().loadTestsFromTestCase(testTemporalBasis)
    return suite

if __name__ == '__main__':
    runner = unittest.TextTestRunner()
    loader = unittest.TestLoader()
    results = unittest.TestResult()
    suite = SuiteTB()
    coverage.erase()
    coverage.start()
    results = runner.run(suite)
    coverage.stop()
    coverage.report('../task.py')
    coverage.erase()





