#! /usr/bin/env python2.5
# Copyright (C) 2008 Martin Albisetti 

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Testing task.WatchFile"""

# standard library imports
import unittest
import sys
import datetime as dt
import random
import sqlite3 as sql
import glob
import os
import os.path
import string
# related 3rd party imports
#import coverage

sys.path.insert(1,sys.path[0].rsplit('/',1)[0])
from task import WatchFile

from_base_dir = '/home/beuno/test/cmp_from/'
to_base_dir = '/home/beuno/test/cmp_to/'
test_from = ('file1', 'file2', 'file3')
test_to = ('file1', 'file2')

extensions = ('xls', 'mp3', 'doc', 'pdf', 'py', 'bin', 'sh', 'html', 'css',
              'js', 'ogg', 'php', 'sql', 'txt', 'xml', 'jpg', 'gif', 'png',
              'tar.gz', 'bz2', 'deb', 'rpm', 'egg', 'pyc', 'java', 'jar',
              'so', 'avi', 'cfg', 'lua', None)
common_files = ('file', 'foo', 'node', 'index', 'page')

working_dir = u"test/dummy/watchfile"

class testError(Exception):
    pass

def give_me_text(min=3, max=30, no_white_space=False):
    """Return a random generated string"""
    val =  ''.join([random.choice(string.letters+string.digits+' ') \
    for i in range(random.randint(min, max))])
    if no_white_space:
        return val.replace(' ', '')
    else:
        return val

def create_file(path):
    """Create a random text file"""
    # sometimes we pick a known name
    if random.randint(0,3) == 0:
        new_file = u"%s%s.%s" % (random.choice(common_files), 
                                 random.randint(1, 1000), 
                                 random.choice(extensions))
    # and sometimes not
    else:
        new_file = u"%s.%s" % (give_me_text(10, 10, True), 
                   random.choice(extensions))
    # if file has no extension, let's get rid of the .
    if new_file.endswith('None'):
        new_file = new_file.rstrip('.None')
    # open the file and edit adding nonsense all the way
    o = open("%s/%s" % (path, new_file), 'w')
    for a in range(1, 5):
        o.write("%s\n" % (give_me_text()))
    o.close()
    return os.path.join(path, new_file)
    
def create_structure():
    """In case it does not exist, create a tree of dirs and files to 
    perform tests. We assume you're running tests at the project root (one
    dir up test)"""

    try:
        os.mkdir(working_dir)
    except OSError, (errno, strerror):
        if errno == 17:
            pass
        else:
            raise testError("%s" % (strerror))
    # decide how many dirs will exist
    dirs = random.randrange(10, 100, 5)
    list_dirs = []
    # inside dummy/watchfile, start creating dirs. First 5 dirs will be 
    # created on root dummy. Then for every new dir, we will choose 1 or 0.
    # if 0 create new dir as child of last dir. If 1 create dir at same level
    # of last dir: apply the same logic until a 5 level depth and then return
    # to surface. Inside every dir we will choose how many files will be
    # created and fill every file with random garbage (just a few lines). Pick
    # the file's extension from "extensions"
    for a in range(1, 6):
        val = unicode(give_me_text(10, 10, True))
        list_dirs.append((val, working_dir))
        os.mkdir(u"%s/%s" % (working_dir, val))
    dirs -= 5
    for b in list_dirs:
        level = 0
        last_dir = os.path.join(b[1], b[0])
        while level < 6 and dirs > 0:
            if last_dir == os.path.join(b[1], b[0]) or \
                            random.randint(0,1) == 0:
                level += 1
                val = unicode(give_me_text(10, 10, True))
                list_dirs.append((val, last_dir))
                os.mkdir(os.path.join(last_dir, val))
                last_dir = os.path.join(last_dir, val)
                dirs -= 1
            else:
                val = unicode(give_me_text(10, 10, True))
                list_dirs.append((val, list_dirs[-1][1]))
                os.mkdir(os.path.join(list_dirs[-1][1], val))
                last_dir = os.path.join(list_dirs[-1][1], val)
                dirs -= 1
    all_files = dict([[x, list()] for x in list_dirs])
    for c in all_files.iterkeys():
        for d in range(0, random.randint(0, 49)):
            all_files[c].append(create_file(os.path.join(c[1], c[0])))
    return all_files

def drop_structure():
    """Erase all garbage files created for these tests"""
    for root, dirs, files in os.walk(working_dir, topdown=False):
        for name in files:
            os.remove(os.path.join(root, name))
        for name in dirs:
            os.rmdir(os.path.join(root, name))

class testWatchFile(unittest.TestCase):
    
    def testCompareTwo(self):
        changed_files = WatchFile(from_base_dir, test_from, 
                                    to_base_dir, test_to)
        print changed_files.get_changed_files()

def SuiteWF():
    suite = unittest.TestLoader().loadTestsFromTestCase(testWatchFile)
    return suite

if __name__ == '__main__':
    file_dict = create_structure()
    runner = unittest.TextTestRunner()
    loader = unittest.TestLoader()
    results = unittest.TestResult()
    suite = SuiteWF()
    #coverage.erase()
    #coverage.start()
    results = runner.run(suite)
    #coverage.stop()
    #coverage.report('../task.py')
    drop_structure()

